# Copyright (C) 2011, Aleksey Lim
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Based on xs-activity-server project code
#
# Copyright (C) 2008 One Laptop Per Child Association, Inc.
# Licensed under the terms of the GNU GPL v2 or later; see COPYING for details.
#
# written by Douglas Bagnall <douglas@paradise.net.nz>

import os
import zipfile
import logging
from cStringIO import StringIO
from ConfigParser import SafeConfigParser
from os.path import join, basename, dirname


_TEMPLATE_DIR = join(dirname(__file__), 'templates')


def bundles(path, locale):
    bundles_list = {}
    for bundle_filename in os.listdir(path):
        bundle_path = join(path, bundle_filename)
        try:
            if bundle_filename.endswith('.xo'):
                bundle = _Activity(bundle_path, locale)
            elif bundle_filename.endswith('.xol'):
                bundle = _Content(bundle_path)
            else:
                continue
            if bundle.bundle_id not in bundles_list or \
                    bundles_list[bundle.bundle_id].version < bundle.version:
                bundles_list[bundle.bundle_id] = bundle
        except Exception, error:
            logging.info('Could not get info about %s: %s', bundle_path, error)

    def read_template(name):
        return file(join(_TEMPLATE_DIR, locale, name)).read()

    page_template = read_template('page')
    activities_template = read_template('activity')
    activities = [bundles_list[i].to_html(activities_template) \
            for i in sorted(bundles_list.keys())]

    return page_template % {'activities': '\n'.join(activities)}


class _Bundle(object):

    def __init__(self, path, info_path, info_section):
        self.zipfile = zipfile.ZipFile(path)
        self.info = self._read_info(info_section, info_path)
        self.info['older_versions'] = ''
        self.info['show_older_versions'] = 'none'
        self.info.setdefault('bundle_url', basename(path))
        self.version = 1
        self.bundle_id = None

    def to_html(self, template):
        self.info.setdefault('description', '')
        self.info['bundle_id'] = self.bundle_id
        self.info['activity_version'] = self.version
        return template % self.info

    def _read_info(self, section, *path):
        root = self.zipfile.namelist()[0].split(os.sep)[0]
        cp = SafeConfigParser()
        try:
            data = self.zipfile.read(join(root, *path))
        except KeyError:
            return {}
        cp.readfp(StringIO(data))
        return dict(cp.items(section))

    def __cmp__(self, other):
        return cmp(self.info.get('name'), other.info.get('name'))


class _Content(_Bundle):

    def __init__(self, path):
        _Bundle.__init__(self, path, 'library/library.info', 'Library')
        self.bundle_id = self.info.get('global_name')
        self.info.setdefault('name', self.bundle_id)
        self.info['description'] = self.info.get('long_name') or ''
        self.version = int(self.info.get('library_version') or 1)


class _Activity(_Bundle):

    def __init__(self, path, locale):
        _Bundle.__init__(self, path, 'activity/activity.info', 'Activity')
        self.info.setdefault('name', self._read_info(
                'Activity', 'locale', locale, 'activity.linfo').get('name'))
        self.bundle_id = self.info.get('bundle_id') or \
                self.info.get('service_name')
        self.version = int(self.info.get('activity_version') or 1)
